/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;


final class JFontChooser extends JDialog {
	private static final boolean DEBUG = false;

	Font					userfont		= new Font("SansSerif", Font.PLAIN, 12);
	boolean				SELECTED		= false;
	
	GraphicsEnvironment	env				= GraphicsEnvironment.getLocalGraphicsEnvironment();
	
	JList					names			= new JList(env.getAvailableFontFamilyNames());
	JList					size			= new JList(new String [] { "6", "8", "10", "12", "14", "16", "18", "20", "24", "36", "48" });
	JTextField				entersize		= new JTextField("") { 
												public Dimension getMaximumSize() {
													return new Dimension(Short.MAX_VALUE, getPreferredSize().height);
												}};
	JFontShower			fontshower		= new JFontShower();
	JButton					ok				= new JButton(ThemeLF_Prefs.GRS("JFontChooser.OK"));
	JButton					reset			= new JButton(ThemeLF_Prefs.GRS("JFontChooser.Reset"));
	JButton					cancel			= new JButton(ThemeLF_Prefs.GRS("JFontChooser.Cancel"));
	
	
	public JFontChooser()  {
		setModal(true);
		
		JPanel contentPane = (JPanel)getContentPane();
		contentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		contentPane.add(createFontPanel());
		contentPane.add(new Strut());
		contentPane.add(new JPopupMenu.Separator());
		contentPane.add(new Strut());
		contentPane.add(createButtons());
		
		pack();
		Dimension size = getSize();
		if (size.width < 300) size.width = 300;
		if (size.height < 300) size.height = 300;
		setSize(size);
	}

	
	private void cancel()  {
		setVisible(false);
	}

	
	private void checkSize()  {
		try
			{
			int size = Integer.parseInt(entersize.getText());
			if (size < 6) { getToolkit().beep(); entersize.setText("6");  return; }
			if (size > 200) { getToolkit().beep(); entersize.setText("200");  return; }
			}
		catch (Exception x) { getToolkit().beep(); entersize.setText("12");  return; }
	}

	
	private Box createButtons()  {
		Box result = Box.createHorizontalBox();
		
		result.add(ok);
		result.add(new Strut());
		result.add(Box.createHorizontalGlue());
		result.add(reset);
		result.add(Box.createHorizontalGlue());
		result.add(new Strut());
		result.add(cancel);
		
		ok.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); }});
		reset.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { reset(); }});
		cancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); }});
		
		return result;
	}

	
	private Font createFont()  {
		checkSize();
		
		return new Font((String)names.getSelectedValue(), Font.PLAIN, Math.abs(Integer.parseInt(entersize.getText())));
	}

	
	private JPanel createFontPanel()  {
		// Gadgets einrichten
		size.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		entersize.setColumns(3);
		names.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		reset();
		
		// Listener
		size.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) return;
				entersize.setText((String)size.getSelectedValue());
				fontshower.setFont(createFont());
			}
		});
		names.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting()) return;
				fontshower.setFont(createFont());
			}
		});
		entersize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				size.setSelectedValue(entersize.getText(), true);
				fontshower.setFont(createFont());
			}
		});
		
		// Listen erzeugen
		Box sizebox = Box.createVerticalBox();
		sizebox.add(new JScrollPane(size, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
		sizebox.add(entersize);
		Box lists = Box.createHorizontalBox();
		lists.add(new JScrollPane(names, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
		lists.add(new Strut());
		lists.add(sizebox);
		
		// Panel erzeugen
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(lists);
		p.add(new Strut());
		p.add(fontshower);
		
		return p;
	}

	
	private void ok()  {
		SELECTED = true;
		userfont = createFont();
		setVisible(false);
	}

	
	private void reset()  {
		size.setSelectedValue(""+userfont.getSize(), true);
		entersize.setText(""+userfont.getSize());
		names.setSelectedValue(userfont.getName(), true);
		fontshower.setFont(userfont);
	}

	
	public Font showDialog(JComponent parent, String title, Font defaultFont)  {
		setTitle(title);
		
		userfont = defaultFont;
		reset();
		
		SELECTED = false;
		setLocation(parent.getLocationOnScreen());
		setVisible(true);
		
		return SELECTED ? userfont : null;
	}

}
