/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.metal.*;
import javax.swing.text.*;


public class ThemeLF extends BasicLookAndFeel {
	private static final boolean DEBUG = false;

	private static Border marginBorder = new BasicBorders.MarginBorder();
	static final Hashtable<String, ThemeLF_Resource> prefs = new Hashtable<String, ThemeLF_Resource>();
	private static String style = ThemeLF_Prefs.DEFAULT_THEME;
	
	
	static {
		if (DEBUG) System.out.println("ThemeLF.StaticInitializer(): installation directory is '"+getInstallationDirectory()+"'");
		
		// The next line is for the font bug :-)
		GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
		
		// load user's prefs
		ThemeLF_Prefs.loadUserData();
	}

	
	private static Object createTextEditorKeyBindings()  {
		return new UIDefaults.LazyInputMap(new Object[] {
			"ctrl C", DefaultEditorKit.copyAction,
			"ctrl V", DefaultEditorKit.pasteAction,
			"ctrl X", DefaultEditorKit.cutAction,
			"COPY", DefaultEditorKit.copyAction,
			"PASTE", DefaultEditorKit.pasteAction,
			"CUT", DefaultEditorKit.cutAction,
			"shift LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift KP_LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift RIGHT", DefaultEditorKit.selectionForwardAction,
			"shift KP_RIGHT", DefaultEditorKit.selectionForwardAction,
			"ctrl LEFT", DefaultEditorKit.previousWordAction,
			"ctrl KP_LEFT", DefaultEditorKit.previousWordAction,
			"ctrl RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl KP_RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl shift LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift KP_LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl shift KP_RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl A", DefaultEditorKit.selectAllAction,
			"HOME", DefaultEditorKit.beginLineAction,
			"END", DefaultEditorKit.endLineAction,
			"shift HOME", DefaultEditorKit.selectionBeginLineAction,
			"shift END", DefaultEditorKit.selectionEndLineAction,
			"UP", DefaultEditorKit.upAction,
			"KP_UP", DefaultEditorKit.upAction,
			"DOWN", DefaultEditorKit.downAction,
			"KP_DOWN", DefaultEditorKit.downAction,
			"PAGE_UP", DefaultEditorKit.pageUpAction,
			"PAGE_DOWN", DefaultEditorKit.pageDownAction,
			"shift PAGE_UP", "selection-page-up",
			"shift PAGE_DOWN", "selection-page-down",
			"ctrl shift PAGE_UP", "selection-page-left",
			"ctrl shift PAGE_DOWN", "selection-page-right",
			"shift UP", DefaultEditorKit.selectionUpAction,
			"shift KP_UP", DefaultEditorKit.selectionUpAction,
			"shift DOWN", DefaultEditorKit.selectionDownAction,
			"shift KP_DOWN", DefaultEditorKit.selectionDownAction,
			"ENTER", DefaultEditorKit.insertBreakAction,
			"typed \010", DefaultEditorKit.deletePrevCharAction,
			"DELETE", DefaultEditorKit.deleteNextCharAction,
			"RIGHT", DefaultEditorKit.forwardAction,
			"LEFT", DefaultEditorKit.backwardAction, 
			"KP_RIGHT", DefaultEditorKit.forwardAction,
			"KP_LEFT", DefaultEditorKit.backwardAction,
			"TAB", DefaultEditorKit.insertTabAction,
			"ctrl BACK_SLASH", "unselect"/*DefaultEditorKit.unselectAction*/,
			"ctrl HOME", DefaultEditorKit.beginAction,
			"ctrl END", DefaultEditorKit.endAction,
			"ctrl shift HOME", DefaultEditorKit.selectionBeginAction,
			"ctrl shift END", DefaultEditorKit.selectionEndAction,
			"ctrl T", "next-link-action",
			"ctrl shift T", "previous-link-action",
			"ctrl SPACE", "activate-link-action",
			"control shift O", "toggle-componentOrientation"/*DefaultEditorKit.toggleComponentOrientation*/
		});
	}

	
	private static Object createTextFieldKeyBindings()  {
		return new UIDefaults.LazyInputMap(new Object[] {
			"ctrl C", DefaultEditorKit.copyAction,
			"ctrl V", DefaultEditorKit.pasteAction,
			"ctrl X", DefaultEditorKit.cutAction,
			"COPY", DefaultEditorKit.copyAction,
			"PASTE", DefaultEditorKit.pasteAction,
			"CUT", DefaultEditorKit.cutAction,
			"shift LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift KP_LEFT", DefaultEditorKit.selectionBackwardAction,
			"shift RIGHT", DefaultEditorKit.selectionForwardAction,
			"shift KP_RIGHT", DefaultEditorKit.selectionForwardAction,
			"ctrl LEFT", DefaultEditorKit.previousWordAction,
			"ctrl KP_LEFT", DefaultEditorKit.previousWordAction,
			"ctrl RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl KP_RIGHT", DefaultEditorKit.nextWordAction,
			"ctrl shift LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift KP_LEFT", DefaultEditorKit.selectionPreviousWordAction,
			"ctrl shift RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl shift KP_RIGHT", DefaultEditorKit.selectionNextWordAction,
			"ctrl A", DefaultEditorKit.selectAllAction,
			"HOME", DefaultEditorKit.beginLineAction,
			"END", DefaultEditorKit.endLineAction,
			"shift HOME", DefaultEditorKit.selectionBeginLineAction,
			"shift END", DefaultEditorKit.selectionEndLineAction,
			"typed \010", DefaultEditorKit.deletePrevCharAction,
			"DELETE", DefaultEditorKit.deleteNextCharAction,
			"RIGHT", DefaultEditorKit.forwardAction,
			"LEFT", DefaultEditorKit.backwardAction,
			"KP_RIGHT", DefaultEditorKit.forwardAction,
			"KP_LEFT", DefaultEditorKit.backwardAction,
			"ENTER", JTextField.notifyAction,
			"ctrl BACK_SLASH", "unselect"/*DefaultEditorKit.unselectAction*/,
			"control shift O", "toggle-componentOrientation"/*DefaultEditorKit.toggleComponentOrientation*/
		});
	}

	
	private static final Object [] getComponentDefaults_Button()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Button");
		
		return new Object [] {
			"Button.background"		, resource.getColor("normalBackgroundColor"),
			"Button.border"			, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_ButtonUI_Border(), marginBorder),
			"Button.font"				, resource.getFont("font"),
			"Button.foreground"		, resource.getColor("textColor"),
			"Button.margin"			, resource.getInsets("margin"),
			"Button.textIconGap"		, new Integer(4),
			"Button.textShiftOffset"		, new Integer(0),
		};
	}

	
	private static final Object [] getComponentDefaults_CheckBox()  {
		ThemeLF_Resource resource = ThemeLF.getUI("CheckBox");
		
		return new Object [] {
			"CheckBox.background"			, resource.getColor("backgroundColor"),
			"CheckBox.border"					, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"CheckBox.font"					, resource.getFont("font"),
			"CheckBox.foreground"				, resource.getColor("textColor"),
			"CheckBox.icon"					, new ThemeLF_CheckBoxUI_CheckIcon(),
			"CheckBox.margin"					, resource.getInsets("margin"),
			"CheckBox.textIconGap"			, new Integer(4),
			"CheckBox.textShiftOffset"			, new Integer(0),
		};
	}

	
	private static final Object [] getComponentDefaults_CheckBoxMenuItem()  {
		ThemeLF_Resource resource = ThemeLF.getUI("MenuItem");
		
		return new Object [] {
			"CheckBoxMenuItem.acceleratorFont"					, resource.getFont("acceleratorFont"),
			"CheckBoxMenuItem.acceleratorForeground"			, resource.getColor("acceleratorTextColor"),
			"CheckBoxMenuItem.acceleratorSelectionForeground"	, resource.getColor("acceleratorSelectionTextColor"),
			"CheckBoxMenuItem.arrowIcon"						, null,
			"CheckBoxMenuItem.background"						, resource.getColor("backgroundColor"),
			"CheckBoxMenuItem.border"							, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_MenuItemUI_Border(), marginBorder),
			"CheckBoxMenuItem.borderPainted"					, Boolean.TRUE,
			"CheckBoxMenuItem.checkIcon"						, new ThemeLF_CheckBoxUI_CheckIcon(),
			"CheckBoxMenuItem.disabledForeground"				, resource.getColor("disabledTextColor"),
			"CheckBoxMenuItem.font"								, resource.getFont("font"),
			"CheckBoxMenuItem.foreground"						, resource.getColor("textColor"),
			"CheckBoxMenuItem.margin"							, resource.getInsets("margin"),
			"CheckBoxMenuItem.selectionBackground"				, resource.getColor("selectionColor"),
			"CheckBoxMenuItem.selectionForeground"				, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_ColorChooser()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ColorChooser");
		
		return new Object [] {
			"ColorChooser.background"					, resource.getColor("backgroundColor"),
			"ColorChooser.font"							, resource.getFont("font"),
			"ColorChooser.foreground"						, resource.getColor("textColor"),
			"ColorChooser.rgbBlueMnemonic"				, new Integer(KeyEvent.VK_B),
			"ColorChooser.rgbGreenMnemonic"				, new Integer(KeyEvent.VK_G),
			"ColorChooser.rgbRedMnemonic"				, new Integer(KeyEvent.VK_R),
			"ColorChooser.swatchesDefaultRecentColor"	, resource.getColor("backgroundColor"),
			"ColorChooser.swatchesRecentSwatchSize"	, new Dimension(10, 10),
			"ColorChooser.swatchesSwatchSize"			, new Dimension(10, 10),
		};
	}

	
	private static final Object [] getComponentDefaults_ComboBox()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ComboBox");
		
		UIManager.put("ComboBox.alternateFocus", new Boolean(resource.getBoolean("alternateFocus")));
		
		return new Object [] {
			// ComboBox
			"ComboBox.background"				, resource.getColor("backgroundColor"),
			"ComboBox.disabledBackground"		, resource.getColor("backgroundColor"),
			"ComboBox.disabledForeground"		, resource.getColor("disabledTextColor"),
			"ComboBox.font"						, resource.getFont("font"),
			"ComboBox.foreground"				, resource.getColor("textColor"),
			"ComboBox.focusColor"					, resource.getColor("focusColor"),
			"ComboBox.selectionBackground"		, resource.getColor("selectionColor"),
			"ComboBox.selectionForeground"		, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_Desktop()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Desktop");
		
		return new Object [] {
			"Desktop.background"	, resource.getColor("desktopColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_DesktopIcon()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Desktop");
		
		return new Object [] {
			// DesktopIcon
			"DesktopIcon.border"		, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("iconBorder"), marginBorder),
			"DesktopIcon.icon"			, resource.getIcon("icon"),
		};
	}

	
	private static final Object [] getComponentDefaults_EditorPane()  {
		ThemeLF_Resource resource = ThemeLF.getUI("EditorPane");
		
		return new Object [] {
			// EditorPane
			"EditorPane.background"			, resource.getColor("backgroundColor"),
			"EditorPane.border"				, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_TextFieldUI_Border(), marginBorder),
			"EditorPane.caretBlinkRate"		, new Integer(resource.getInteger("caretBlinkRate")),
			"EditorPane.caretForeground"		, resource.getColor("caretColor"),
			"EditorPane.font"					, resource.getFont("font"),
			"EditorPane.foreground"			, resource.getColor("textColor"),
			"EditorPane.inactiveForeground"	, resource.getColor("textColor"),
			"EditorPane.focusInputMap"		, createTextEditorKeyBindings(),
			"EditorPane.margin"				, resource.getInsets("margin"),
			"EditorPane.selectionBackground"	, resource.getColor("selectionColor"),
			"EditorPane.selectionForeground"	, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_FileChooser()  {
		ThemeLF_Resource resource = ThemeLF.getUI("FileChooser");
		
		return new Object [] {
			"FileChooser.detailsViewIcon"				, resource.getIcon("detailsViewIcon"),
			"FileChooser.homeFolderIcon"				, resource.getIcon("homeFolderIcon"),
			"FileChooser.listViewIcon"					, resource.getIcon("listViewIcon"),
			"FileChooser.newFolderIcon"				, resource.getIcon("newFolderIcon"),
			"FileChooser.upFolderIcon"					, resource.getIcon("upFolderIcon"),
			"FileChooser.cancelButtonMnemonic"		, new Integer(KeyEvent.VK_C),
			"FileChooser.helpButtonMnemonic"			, new Integer(KeyEvent.VK_H),
			"FileChooser.openButtonMnemonic"		, new Integer(KeyEvent.VK_O),
			"FileChooser.saveButtonMnemonic"			, new Integer(KeyEvent.VK_S),
			"FileChooser.updateButtonMnemonic"		, new Integer(KeyEvent.VK_U),
			"FileView.computerIcon"					, resource.getIcon("computerIcon"),
			"FileView.directoryIcon"						, resource.getIcon("directoryIcon"),
			"FileView.fileIcon"							, resource.getIcon("fileIcon"),
			"FileView.floppyDriveIcon"					, resource.getIcon("floppyDriveIcon"),
			"FileView.hardDriveIcon"					, resource.getIcon("hardDriveIcon"),
		};
	}

	
	private static final Object [] getComponentDefaults_InternalFrame()  {
		ThemeLF_Resource resource = ThemeLF.getUI("InternalFrame");
		
		return new Object [] {
			"InternalFrame.activeTitleBackground"		, resource.getColor("titleActiveColor"),
			"InternalFrame.activeTitleForeground"		, resource.getColor("titleFontColor"),
			"InternalFrame.border"						, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_InternalFrameUI_Border(), marginBorder),
			"InternalFrame.closeIcon"					, resource.getIcon("closeIcon"),
			"InternalFrame.closeTip"					, resource.getText("closeTip"),
			"InternalFrame.icon"						, null,
			"InternalFrame.iconifyIcon"					, resource.getIcon("iconifyIcon"),
			"InternalFrame.iconifyTip"					, resource.getText("iconifyTip"),
			"InternalFrame.inactiveTitleBackground"	, resource.getColor("backgroundColor"), 
			"InternalFrame.inactiveTitleForeground"	, resource.getColor("titleFontColor"),
			"InternalFrame.maximizeIcon"				, resource.getIcon("maximizeIcon"),
			"InternalFrame.maximizeTip"				, resource.getText("maximizeTip"),
		//	"InternalFrame.minimizeIcon"				, resource.getIcon("minimizeIcon"),
			"InternalFrame.resizeTip"					, resource.getText("resizeTip"),
			"InternalFrame.shadedIcon"				, resource.getIcon("shadedIcon"),
			"InternalFrame.titleFont"					, resource.getFont("titleFont")
		};
	}

	
	private static final Object [] getComponentDefaults_Label(UIDefaults table)  {
		ThemeLF_Resource resource = ThemeLF.getUI("Label");
		
		return new Object [] {
			"Label.background"				, resource.getColor("backgroundColor"),
			"Label.border"					, null,
			"Label.disabledForeground"	, resource.getColor("disabledTextColor"),
			"Label.disabledShadow"		, table.get("controlShadow"),
			"Label.font"					, resource.getFont("font"),
			"Label.foreground"				, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_List()  {
		ThemeLF_Resource resource = ThemeLF.getUI("List");
		
		return new Object [] {
			"List.background"					, resource.getColor("backgroundColor"),
			"List.border"						, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_ListUI_Border(), marginBorder),
			"List.focusCellHighlightBorder"		, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("cellHighlightBorder"), marginBorder),
			"List.font"							, resource.getFont("font"),
			"List.foreground"					, resource.getColor("textColor"),
			"List.selectionBackground"			, resource.getColor("selectionColor"),
			"List.selectionForeground"			, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_Menu()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Menu");
		
		return new Object [] {
			"Menu.acceleratorFont"						, resource.getFont("acceleratorFont"),
			"Menu.acceleratorForeground"				, resource.getColor("acceleratorTextColor"),
			"Menu.acceleratorSelectionForeground"	, resource.getColor("acceleratorSelectionTextColor"),
		//	"Menu.arrowIcon"							, ThreeDPrefs.menuArrowIcon,
			"Menu.background"							, resource.getColor("backgroundColor"),
			"Menu.border"								, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_MenuUI_Border(), marginBorder),
			"Menu.borderPainted"						, Boolean.TRUE,
			"Menu.checkIcon"							, null,
			"Menu.consumesTabs"						, Boolean.TRUE,
			"Menu.disabledForeground"				, resource.getColor("disabledTextColor"),
			"Menu.font"								, resource.getFont("font"),
			"Menu.foreground"							, resource.getColor("textColor"),
			"Menu.margin"								, resource.getInsets("margin"),
			"Menu.selectionBackground"				, resource.getColor("selectionColor"),
			"Menu.selectionForeground"				, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_MenuBar()  {
		ThemeLF_Resource resource = ThemeLF.getUI("MenuBar");
		
		return new Object [] {
			"MenuBar.background"	, resource.getColor("backgroundColor"),
			"MenuBar.border"		, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_MenuBarUI_Border(), marginBorder),
			"MenuBar.font"			, resource.getFont("font"),
			"MenuBar.foreground"	, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_MenuItem()  {
		ThemeLF_Resource resource = ThemeLF.getUI("MenuItem");
		
		return new Object [] {
			"MenuItem.acceleratorDelimiter"				, "\u00b7",
			"MenuItem.acceleratorFont"					, resource.getFont("acceleratorFont"),
			"MenuItem.acceleratorForeground"				, resource.getColor("acceleratorTextColor"),
			"MenuItem.acceleratorSelectionForeground"	, resource.getColor("acceleratorSelectionTextColor"),
			"MenuItem.arrowIcon"							, null,
			"MenuItem.background"						, resource.getColor("backgroundColor"),
			"MenuItem.border"								, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_MenuItemUI_Border(), marginBorder),
			"MenuItem.borderPainted"						, Boolean.TRUE,
			"MenuItem.checkIcon"							, null,
			"MenuItem.disabledForeground"				, resource.getColor("disabledTextColor"),
			"MenuItem.font"								, resource.getFont("font"),
			"MenuItem.foreground"							, resource.getColor("textColor"),
			"MenuItem.margin"								, resource.getInsets("margin"),
			"MenuItem.selectionBackground"				, resource.getColor("selectionColor"),
			"MenuItem.selectionForeground"				, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_OptionPane()  {
		ThemeLF_Resource resource = ThemeLF.getUI("OptionPane");
		
		return new Object [] {
			"OptionPane.background"			, resource.getColor("backgroundColor"),
			"OptionPane.border"				, new BorderUIResource.EmptyBorderUIResource(10, 10, 10, 10),
			"OptionPane.buttonAreaBorder"	, new BorderUIResource.EmptyBorderUIResource(5,15,5,0),
			"OptionPane.errorIcon"				, resource.getIcon("errorIcon"),
			"OptionPane.font"					, resource.getFont("font"),
			"OptionPane.foreground"			, resource.getColor("textColor"),
			"OptionPane.informationIcon"		, resource.getIcon("informationIcon"),
			"OptionPane.messageAreaBorder"	, new BorderUIResource.EmptyBorderUIResource(0,0,0,0),
			"OptionPane.messageForeground"	, resource.getColor("messageTextColor"),
			"OptionPane.minimumSize"			, new DimensionUIResource(262, 90),
			"OptionPane.questionIcon"			, resource.getIcon("questionIcon"),
			"OptionPane.warningIcon"			, resource.getIcon("warningIcon"),
		};
	}

	
	private static final Object [] getComponentDefaults_Panel()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Panel");
		
		return new Object [] {
			"Panel.background"	, resource.getColor("backgroundColor"),
			"Panel.font"			, resource.getFont("font"),
			"Panel.foreground"		, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_PasswordField()  {
		ThemeLF_Resource resource = ThemeLF.getUI("TextField");
		
		return new Object [] {
			"PasswordField.background"			, resource.getColor("backgroundColor"),
			"PasswordField.border"					, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"PasswordField.caretBlinkRate"			, new Integer(resource.getInteger("caretBlinkRate")),
			"PasswordField.caretForeground"		, resource.getColor("caretColor"),
			"PasswordField.font"					, resource.getFont("font"),
			"PasswordField.foreground"			, resource.getColor("textColor"),
			"PasswordField.inactiveForeground"	, resource.getColor("textColor"),
			"PasswordField.focusInputMap"		, createTextFieldKeyBindings(),
			"PasswordField.margin"				, resource.getInsets("margin"),
			"PasswordField.selectionBackground"	, resource.getColor("selectionColor"),
			"PasswordField.selectionForeground"	, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_PopupMenu()  {
		ThemeLF_Resource resource = ThemeLF.getUI("PopupMenu");
		
		return new Object [] {
			"PopupMenu.background"	, resource.getColor("backgroundColor"),
			"PopupMenu.border"		, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_PopupMenuUI_Border(), marginBorder),          
			"PopupMenu.font"			, resource.getFont("font"),
			"PopupMenu.foreground"	, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_PopupMenuSeparator()  {
		ThemeLF_Resource resource = ThemeLF.getUI("PopupMenu");
		
		return new Object [] {
			"PopupMenu.background"	, resource.getColor("backgroundColor"),
			"PopupMenu.border"		, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),          
			"PopupMenu.font"			, resource.getFont("font"),
			"PopupMenu.foreground"	, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_ProgressBar()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ProgressBar");
		
		return new Object [] {
			"ProgressBar.background"				, resource.getColor("backgroundColor"),
			"ProgressBar.border"					, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"ProgressBar.cellLength"				, new Integer(1),
			"ProgressBar.cellSpacing"				, new Integer(0),
			"ProgressBar.font"						, resource.getFont("font"),
			"ProgressBar.foreground"				, resource.getColor("selectionColor"), 
			"ProgressBar.selectionBackground"		, resource.getColor("textColor"), 
			"ProgressBar.selectionForeground"		, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_RadioButton()  {
		ThemeLF_Resource resource = ThemeLF.getUI("RadioButton");
		
		return new Object [] {
			// RadioButton
			"RadioButton.background"		, resource.getColor("backgroundColor"),
			"RadioButton.border"			, BorderFactory.createEmptyBorder(1,1,1,1), // wegen des Focus-Rahmens
			"RadioButton.font"				, resource.getFont("font"),
			"RadioButton.foreground"		, resource.getColor("textColor"),
			"RadioButton.icon"				, new ThemeLF_RadioButtonUI_RadioIcon(),
			"RadioButton.margin"			, resource.getInsets("margin"),
			"RadioButton.textIconGap"		, new Integer(4),
			"RadioButton.textShiftOffset"	, new Integer(0),
		};
	}

	
	private static final Object [] getComponentDefaults_RadioButtonMenuItem()  {
		ThemeLF_Resource resource = ThemeLF.getUI("MenuItem");
		
		return new Object [] {
			"RadioButtonMenuItem.acceleratorFont"				, resource.getFont("acceleratorFont"),
			"RadioButtonMenuItem.acceleratorForeground"			, resource.getColor("acceleratorTextColor"),
			"RadioButtonMenuItem.acceleratorSelectionForeground", resource.getColor("acceleratorTextColor"),
			"RadioButtonMenuItem.arrowIcon"						, null,
			"RadioButtonMenuItem.background"					, resource.getColor("backgroundColor"),
			"RadioButtonMenuItem.border"							, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_MenuItemUI_Border(), marginBorder),
			"RadioButtonMenuItem.borderPainted"					, Boolean.TRUE,
			"RadioButtonMenuItem.checkIcon"						, new ThemeLF_RadioButtonUI_RadioIcon(),
			"RadioButtonMenuItem.disabledForeground"			, resource.getColor("disabledTextColor"),
			"RadioButtonMenuItem.font"							, resource.getFont("font"),
			"RadioButtonMenuItem.foreground"					, resource.getColor("textColor"),
			"RadioButtonMenuItem.margin"							, resource.getInsets("margin"),
			"RadioButtonMenuItem.selectionBackground"			, resource.getColor("selectionColor"),
			"RadioButtonMenuItem.selectionForeground"			, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_ScrollBar()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ScrollBar");
		
		UIManager.put("ScrollBar.alternateLayout", new Boolean(resource.getBoolean("alternateLayout")));
		
		return new Object [] {
			"ScrollBar.background"			, resource.getColor("backgroundColor"),
			"ScrollBar.border"				, null,
			"ScrollBar.foreground"			, resource.getColor("textColor"),
			"ScrollBar.lighShadow"			, Color.red,
			"ScrollBar.maximumThumbSize"	, new Dimension(1024,1024),
			"ScrollBar.minimumThumbSize"	, new Dimension(17,17),
			"ScrollBar.thumb"				, Color.red,
			"ScrollBar.thumbDarkShadow"	, Color.red,
			"ScrollBar.thumbHighlight"		, Color.red,
			"ScrollBar.track"				, Color.red,
			"ScrollBar.trackHighlight"		, Color.red,
		};
	}

	
	private static final Object [] getComponentDefaults_ScrollPane()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ScrollPane");
		
		return new Object [] {
			// ScrollPane
			"ScrollPane.background"		, resource.getColor("backgroundColor"),
			"ScrollPane.border"				, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_ScrollPaneUI_Border(), marginBorder),
			"ScrollPane.font"				, resource.getFont("font"),
			"ScrollPane.foreground"		, resource.getColor("textColor"),
			"ScrollPane.viewportBorder"	, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_ScrollPaneUI_ViewportBorder(), marginBorder),
		};
	}

	
	private static final Object [] getComponentDefaults_Separator()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Separator");
		
		return new Object [] {
			"Separator.background"	, resource.getColor("lightEdgeColor"),
			"Separator.foreground"		, resource.getColor("darkEdgeColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_Slider()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Slider");
		
		return new Object [] {
			// Slider
			"Slider.background"	, resource.getColor("backgroundColor"),
			"Slider.border"			, null,
			"Slider.focus"			, resource.getColor("focusColor"),
			"Slider.focusInsets"		, new InsetsUIResource(1,1,1,1),
			"Slider.foreground"		, resource.getColor("textColor"),
			"Slider.highlight"		, new Color(100,20,100),
			"Slider.shadow"		, new Color(20,50,130),
		};
	}

	
	private static final Object [] getComponentDefaults_SplitPane()  {
		ThemeLF_Resource resource = ThemeLF.getUI("SplitPane");
		
		return new Object [] {
			"SplitPane.background"		, resource.getColor("backgroundColor"),
			"SplitPane.border"			, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_SplitPaneUI_Border(), marginBorder),
			"SplitPaneDivider.border"	, null,
		//	"SplitPane.dividerSize"		, new Integer(resource.getInteger("dividerSize")),
		//	"SplitPane.hightlight"		, resource.getColor("lightEdgeColor"),
		//	"SplitPane.shadow"		, resource.getColor("darkEdgeColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_TabbedPane()  {
		ThemeLF_Resource resource = ThemeLF.getUI("TabbedPane");
		
		return new Object [] {
			"TabbedPane.background"				, resource.getColor("backgroundColor"),
			"TabbedPane.contentBorderInsets"	, resource.getInsets("contentBorderInsets"),
			"TabbedPane.darkShadow"				, resource.getColor("darkShadowColor"),
			"TabbedPane.focus"					, resource.getColor("focusColor"),
			"TabbedPane.font"						, resource.getFont("font"),
			"TabbedPane.foreground"				, resource.getColor("textColor"),
			"TabbedPane.highlight"					, resource.getColor("darkShadowColor"),
			"TabbedPane.lightHighlight"			, resource.getColor("darkShadowColor"),
			"TabbedPane.selectedTabPadInsets"	, resource.getInsets("selectedTabPadInsets"),
			"TabbedPane.shadow"					, resource.getColor("darkShadowColor"),
			"TabbedPane.tabAreaInsets"			, resource.getInsets("tabAreaInsets"),
			"TabbedPane.tabInsets"				, resource.getInsets("tabInsets"),
			"TabbedPane.tabRunOverlay"			, new Integer(2),
			"TabbedPane.textIconGap"				, new Integer(4),
		};
	}

	
	private static final Object [] getComponentDefaults_Table()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Table");
		
		return new Object [] {
			"Table.background"					, resource.getColor("backgroundColor"),
			"Table.focusCellBackground"		, resource.getColor("backgroundColor"),
			"Table.focusCellForeground"		, resource.getColor("textColor"),
			"Table.focusCellHighlightBorder"	, new BorderUIResource.LineBorderUIResource(resource.getColor("focusColor")),
			"Table.font"						, resource.getFont("font"),
			"Table.foreground"					, resource.getColor("textColor"),
			"Table.gridColor"					, resource.getColor("backgroundColor").darker(),
			"Table.scrollPaneBorder"			, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_TableUI_Border(), marginBorder),
			"Table.selectionBackground"		, resource.getColor("selectionColor"),
			"Table.selectionForeground"		, resource.getColor("selectionTextColor"),
			"TableHeader.background"			, resource.getColor("headerBackgroundColor"),
			"TableHeader.cellBorder"			, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_TableHeaderUI_Border(), marginBorder),
			"TableHeader.font"					, resource.getFont("font"),
			"TableHeader.foreground"			, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_TextArea()  {
		ThemeLF_Resource resource = ThemeLF.getUI("TextArea");
		
		return new Object [] {
			"TextArea.background"				, resource.getColor("backgroundColor"),
			"TextArea.border"					, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_TextFieldUI_Border(), marginBorder),
			"TextArea.caretBlinkRate"			, new Integer(resource.getInteger("caretBlinkRate")),
			"TextArea.caretForeground"		, resource.getColor("caretColor"),
			"TextArea.font"						, resource.getFont("font"),
			"TextArea.foreground"				, resource.getColor("textColor"),
			"TextArea.inactiveForeground"		, resource.getColor("textColor"),
			"TextArea.focusInputMap"			, createTextEditorKeyBindings(),
			"TextArea.margin"					, resource.getInsets("margin"),
			"TextArea.selectionBackground"	, resource.getColor("selectionColor"),
			"TextArea.selectionForeground"	, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_TextField()  {
		ThemeLF_Resource resource = ThemeLF.getUI("TextField");
		
		return new Object [] {
			"TextField.background"				, resource.getColor("backgroundColor"),
			"TextField.border"					, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_TextFieldUI_Border(), marginBorder),
			"TextField.caretBlinkRate"			, new Integer(resource.getInteger("caretBlinkRate")),
			"TextField.caretForeground"		, resource.getColor("caretColor"),
			"TextField.font"						, resource.getFont("font"),
			"TextField.foreground"				, resource.getColor("textColor"),
			"TextField.inactiveForeground"		, resource.getColor("textColor"),
			"TextField.focusInputMap"			, createTextFieldKeyBindings(),
			"TextField.margin"					, resource.getInsets("margin"),
			"TextField.selectionBackground"	, resource.getColor("selectionColor"),
			"TextField.selectionForeground"	, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_TextPane()  {
		ThemeLF_Resource resource = ThemeLF.getUI("EditorPane");
		
		return new Object [] {
			"TextPane.background"				, resource.getColor("backgroundColor"),
			"TextPane.border"					, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_TextFieldUI_Border(), marginBorder),
			"TextPane.caretBlinkRate"			, new Integer(resource.getInteger("caretBlinkRate")),
			"TextPane.caretForeground"		, resource.getColor("caretColor"),
			"TextPane.font"					, resource.getFont("font"),
			"TextPane.foreground"				, resource.getColor("textColor"),
			"TextPane.inactiveForeground"		, resource.getColor("textColor"),
			"TextPane.focusInputMap"			, createTextEditorKeyBindings(),
			"TextPane.margin"					, resource.getInsets("margin"),
			"TextPane.selectionBackground"	, resource.getColor("selectionColor"),
			"TextPane.selectionForeground"	, resource.getColor("selectionTextColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_TitledBorder()  {
		ThemeLF_Resource resource = ThemeLF.getUI("TitledBorder");
		
		return new Object [] {
			"TitledBorder.border"	, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"TitledBorder.font"		, resource.getFont("font"),
			"TitledBorder.titleColor"	, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_ToggleButton()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Button");
		
		return new Object [] {
			"ToggleButton.background"		, resource.getColor("normalBackgroundColor"),
			"ToggleButton.border"			, new BorderUIResource.CompoundBorderUIResource(new ThemeLF_ToggleButtonUI_Border(), marginBorder),
			"ToggleButton.font"			, resource.getFont("font"),
			"ToggleButton.foreground"		, resource.getColor("textColor"),
			"ToggleButton.margin"			, resource.getInsets("margin"),
			"ToggleButton.textIconGap"	, new Integer(4),
			"ToggleButton.textShiftOffset"	, new Integer(0),
		};
	}

	
	private static final Object [] getComponentDefaults_ToolBar()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ToolBar");
		
		return new Object [] {
			"ToolBar.background"			, resource.getColor("backgroundColor"),
			"ToolBar.border"				, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"ToolBar.dockingBackground"	, resource.getColor("dockingBackgroundColor"),
			"ToolBar.dockingForeground"	, resource.getColor("textColor"),
			"ToolBar.floatingBackground"	, resource.getColor("floatingBackgroundColor"),
			"ToolBar.floatingForeground"	, resource.getColor("floatingTextColor"),
			"ToolBar.font"					, resource.getFont("font"),
			"ToolBar.foreground"			, resource.getColor("textColor"),
			"ToolBar.separatorSize"		, new DimensionUIResource(10,10),
		};
	}

	
	private static final Object [] getComponentDefaults_ToolTip()  {
		ThemeLF_Resource resource = ThemeLF.getUI("ToolTip");
		
		return new Object [] {
			"ToolTip.background"	, resource.getColor("backgroundColor"),
			"ToolTip.border"		, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"ToolTip.font"			, resource.getFont("font"),
			"ToolTip.foreground"	, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_Tree()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Tree");
		
		return new Object [] {
			"Tree.background"						, resource.getColor("backgroundColor"),
			"Tree.changeSelectionWithFocus"		, Boolean.TRUE,
			"Tree.closedIcon"						, resource.getIcon("closedIcon"),
			"Tree.collapsedIcon"					, resource.getIcon("collapsedIcon"),
			"Tree.drawsFocusBorderAroundIcon"	, Boolean.FALSE,
			"Tree.editorBorder"						, new BorderUIResource.CompoundBorderUIResource(resource.getBorder("border"), marginBorder),
			"Tree.expandedIcon"					, resource.getIcon("expandedIcon"),
			"Tree.font"								, resource.getFont("font"),
			"Tree.foreground"						, resource.getColor("textColor"),
			"Tree.hash"							, resource.getColor("treeHashColor"),
			"Tree.leafIcon"							, resource.getIcon("leafIcon"),
			"Tree.leftChildIndent"					, new Integer(7),
			"Tree.openIcon"						, resource.getIcon("openIcon"),
			"Tree.rightChildIndent"					, new Integer(13),
			"Tree.rowHeight"						, new Integer(16),
			"Tree.scrollsOnExpand"					, Boolean.TRUE,
			"Tree.selectionBackground"				, resource.getColor("selectionColor"),
			"Tree.selectionBorderColor"			, resource.getColor("focusColor"),
			"Tree.selectionForeground"				, resource.getColor("selectionTextColor"),
			"Tree.textBackground"					, resource.getColor("backgroundColor"),
			"Tree.textForeground"					, resource.getColor("textColor"),
		};
	}

	
	private static final Object [] getComponentDefaults_Viewport()  {
		ThemeLF_Resource resource = ThemeLF.getUI("Viewport");
		
		return new Object [] {
			"Viewport.background"	, resource.getColor("backgroundColor"),
			"Viewport.font"			, resource.getFont("font"),
			"Viewport.foreground"	, resource.getColor("textColor")
		};
	}

	
	public static final FontUIResource getDefaultFont(String s)  {
		ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
		if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseColors(): cannot find Defaults file");
		return resource.getFont(s);
	}

	
	public UIDefaults getDefaults()  {
		UIDefaults table = super.getDefaults();
		
		return table;
	}

	
	public String getDescription()  {
		return "Theme Look & Feel v1.3";
	}

	
	public String getID()  {
		return "ThemeLF";
	}

	
	public static String getInstallationDirectory()  {
		String [] dirs  = {
			"/opt/ThemeLF",
			"/opt2/ThemeLF",
			"/usr/ThemeLF",
			"/usr/local/ThemeLF",
			"/usr/lib/ThemeLF",
			"/usr/local/lib/ThemeLF",
			System.getProperty("user.home") + File.separator + "ThemeLF",
			"C:\\Program Files\\ThemeLF",
			"D:\\Program Files\\ThemeLF",
			"E:\\Program Files\\ThemeLF",
			"C:\\Programme\\ThemeLF",
			"D:\\Programme\\ThemeLF",
			"E:\\Programme\\ThemeLF",
			"C:\\Java\\ThemeLF",
			"D:\\Java\\ThemeLF",
			"E:\\Java\\ThemeLF",
		};
		
		for (int i=0; i<dirs.length; ++i)
			{
			File f = new File(dirs[i]);
			if (f.exists()) return dirs[i];
			}
		
		ThemeLF_Utils.showError("ThemeLF has not been installed correctly. See docs for further information.");
		System.exit(10);
		
		return null;
	}

	
	public String getName()  {
		return "ThemeLF";
	}

	
	public static String [] getThemes()  {
		File themeDir = new File(ThemeLF.getInstallationDirectory(), "data" + File.separator + "ui");
		return themeDir.list(new FilenameFilter() { public boolean accept(File dir, String name) { return new File(dir, name).isDirectory(); }});
	}

	
	public static final ThemeLF_Resource getUI(String id)  {
		if (! style.equals(ThemeLF_Prefs.getStyle()))
			{
			style = ThemeLF_Prefs.getStyle();
			prefs.clear();
			}
		
		ThemeLF_Resource o = prefs.get(id);
		if (o == null) 
			{
			if (DEBUG) System.out.println("ThemeLF.getUI(): creating new UI for " + id);
			o = new ThemeLF_Resource(style, id);
			prefs.put(id, o);
			}
		
		return o;
	}

	/**
	* Initialize the uiClassID to BasicComponentUI mapping.
	* The JComponent classes define their own uiClassID constants
	* (see AbstractComponent.getUIClassID).  This table must
	* map those constants to a BasicComponentUI class of the
	* appropriate type.
	* 
	* @see BasicLookAndFeel#getDefaults
	* 
	*/

	
	protected void initClassDefaults(UIDefaults table)  {
		if (DEBUG) System.out.println("ThemeLF.initClassDefaults()");
		
		super.initClassDefaults(table);
		
		Object[] uiDefaults = {
			"ButtonUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ButtonUI",
			"CheckBoxMenuItemUI"		, "de.hillenbrand.swing.plaf.themelf.ThemeLF_CheckBoxMenuItemUI",
			"CheckBoxUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_CheckBoxUI",
			"ColorChooserUI"			, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ColorChooserUI",
			"ComboBoxUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ComboBoxUI",
			"DesktopIconUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_DesktopIconUI",
			"DesktopPaneUI"			, "de.hillenbrand.swing.plaf.themelf.ThemeLF_DesktopPaneUI",
			"EditorPaneUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_EditorPaneUI",
			"FileChooserUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_FileChooserUI",
			"InternalFrameUI"			, "de.hillenbrand.swing.plaf.themelf.ThemeLF_InternalFrameUI",
			"LabelUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_LabelUI",
			"ListUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ListUI",
			"MenuBarUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_MenuBarUI",
			"MenuItemUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_MenuItemUI",
			"MenuUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_MenuUI",
			"OptionPaneUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_OptionPaneUI",
			"PanelUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_PanelUI",
			"PasswordFieldUI"			, "de.hillenbrand.swing.plaf.themelf.ThemeLF_PasswordFieldUI",
			"PopupMenuSeparatorUI"	, "de.hillenbrand.swing.plaf.themelf.ThemeLF_PopupMenuSeparatorUI",
			"PopupMenuUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_PopupMenuUI",
			"ProgressBarUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ProgressBarUI",
			"RadioButtonMenuItemUI"	, "de.hillenbrand.swing.plaf.themelf.ThemeLF_RadioButtonMenuItemUI",
			"RadioButtonUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_RadioButtonUI",
			"ScrollBarUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ScrollBarUI",
			"ScrollPaneUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ScrollPaneUI",
			"SeparatorUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_SeparatorUI",
			"SliderUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_SliderUI",
			"SplitPaneUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_SplitPaneUI",
			"TabbedPaneUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TabbedPaneUI",
			"TableHeaderUI"				, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TableHeaderUI",
			"TableUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TableUI",
			"TextAreaUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TextAreaUI",
			"TextFieldUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TextFieldUI",
			"TextPaneUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TextPaneUI",
			"ToggleButtonUI"			, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ToggleButtonUI",
			"ToolBarSeparatorUI"		, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ToolBarSeparatorUI",
			"ToolBarUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ToolBarUI",
			"ToolTipUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ToolTipUI",
			"TreeUI"						, "de.hillenbrand.swing.plaf.themelf.ThemeLF_TreeUI",
			"ViewportUI"					, "de.hillenbrand.swing.plaf.themelf.ThemeLF_ViewportUI",
		};
		
		table.putDefaults(uiDefaults);
	}

	
	protected void initComponentDefaults(UIDefaults table)  {
		if (DEBUG) System.out.println("ThemeLF.initComponentDefaults()");
		
		super.initComponentDefaults(table);
		
		table.putDefaults(ThemeLF.getComponentDefaults_Button());
		table.putDefaults(ThemeLF.getComponentDefaults_CheckBox());
		table.putDefaults(ThemeLF.getComponentDefaults_CheckBoxMenuItem());
		table.putDefaults(ThemeLF.getComponentDefaults_ColorChooser());
		table.putDefaults(ThemeLF.getComponentDefaults_ComboBox());
		table.putDefaults(ThemeLF.getComponentDefaults_Desktop());
		table.putDefaults(ThemeLF.getComponentDefaults_DesktopIcon());
		table.putDefaults(ThemeLF.getComponentDefaults_EditorPane());
		table.putDefaults(ThemeLF.getComponentDefaults_FileChooser());
		table.putDefaults(ThemeLF.getComponentDefaults_InternalFrame());
		table.putDefaults(ThemeLF.getComponentDefaults_Label(table));
		table.putDefaults(ThemeLF.getComponentDefaults_List());
		table.putDefaults(ThemeLF.getComponentDefaults_Menu());
		table.putDefaults(ThemeLF.getComponentDefaults_MenuBar());
		table.putDefaults(ThemeLF.getComponentDefaults_MenuItem());
		table.putDefaults(ThemeLF.getComponentDefaults_OptionPane());
		table.putDefaults(ThemeLF.getComponentDefaults_Panel());
		table.putDefaults(ThemeLF.getComponentDefaults_PasswordField());
		table.putDefaults(ThemeLF.getComponentDefaults_PopupMenu());
		table.putDefaults(ThemeLF.getComponentDefaults_ProgressBar());
		table.putDefaults(ThemeLF.getComponentDefaults_RadioButton());
		table.putDefaults(ThemeLF.getComponentDefaults_RadioButtonMenuItem());
		table.putDefaults(ThemeLF.getComponentDefaults_ScrollBar());
		table.putDefaults(ThemeLF.getComponentDefaults_ScrollPane());
		table.putDefaults(ThemeLF.getComponentDefaults_Separator());
		table.putDefaults(ThemeLF.getComponentDefaults_Slider());
		table.putDefaults(ThemeLF.getComponentDefaults_SplitPane());
		table.putDefaults(ThemeLF.getComponentDefaults_TabbedPane());
		table.putDefaults(ThemeLF.getComponentDefaults_Table());
		table.putDefaults(ThemeLF.getComponentDefaults_TextArea());
		table.putDefaults(ThemeLF.getComponentDefaults_TextField());
		table.putDefaults(ThemeLF.getComponentDefaults_TextPane());
		table.putDefaults(ThemeLF.getComponentDefaults_TitledBorder());
		table.putDefaults(ThemeLF.getComponentDefaults_ToggleButton());
		table.putDefaults(ThemeLF.getComponentDefaults_ToolBar());
		table.putDefaults(ThemeLF.getComponentDefaults_ToolTip());
		table.putDefaults(ThemeLF.getComponentDefaults_Tree());
		table.putDefaults(ThemeLF.getComponentDefaults_Viewport());
	}

	/**
	* Load the SystemColors into the defaults table.  The keys
	* for SystemColor defaults are the same as the names of
	* the public fields in SystemColor.
	* 
	*/

	
	protected void initSystemColorDefaults(UIDefaults table)  {
		super.initSystemColorDefaults(table);
		
		ThemeLF_Resource desktopResource		= getUI("Desktop");
		
		Object[] systemColors = {
			"desktop"	, desktopResource.getColor("desktopColor"),
			"window"	, desktopResource.getColor("desktopColor"),
		};
		
		for (int i = 0; i < systemColors.length; i += 2) table.put((String)systemColors[i], systemColors[i + 1]);
	}

	
	public boolean isNativeLookAndFeel()  {
		return false;
	}

	
	public boolean isSupportedLookAndFeel()  {
		return true;
	}

}
