/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


public class ThemeLF_CheckBoxUI_CheckIcon implements Icon, UIResource {
	private static final boolean DEBUG = false;

	ThemeLF_Resource	resource 						= ThemeLF.getUI("CheckBox");
	
	Border 				iconBorder 						= resource.getBorder("iconBorder");
	Border 				pressedIconBorder 				= resource.getBorder("pressedIconBorder");
	
	Color				iconBackgroundColor				= resource.getColor("iconBackgroundColor");
	Color				pressedIconBackgroundColor		= resource.getColor("pressedIconBackgroundColor");
	
	Icon 				icon 							= resource.getIcon("icon");
	Icon 				selectedIcon 					= resource.getIcon("selectedIcon");
	
	Insets				borderInsets						= iconBorder.getBorderInsets(null);
	Insets				iconInsets						= resource.getInsets("iconInsets");
	
	
	public int getIconHeight()  {
		return selectedIcon.getIconHeight() + iconInsets.top + iconInsets.bottom + borderInsets.top + borderInsets.bottom;
	}

	
	public int getIconWidth()  {
		return selectedIcon.getIconWidth() + iconInsets.left + iconInsets.right + borderInsets.left + borderInsets.right;
	}

	
	public void paintIcon(Component c, Graphics g, int x, int y)  {
		AbstractButton cb = (AbstractButton)c;
		ButtonModel model = cb.getModel();
		
		if (model.isPressed())
			{
			g.setColor(pressedIconBackgroundColor);
			g.fillRect(x, y, getIconWidth()-1, getIconHeight()-1);
			pressedIconBorder.paintBorder(c, g, x, y, getIconWidth(), getIconHeight());
			}
		else 
			{
			g.setColor(iconBackgroundColor);
			g.fillRect(x, y, getIconWidth()-1, getIconHeight()-1);
			iconBorder.paintBorder(c, g, x, y, getIconWidth(), getIconHeight());
			}
		
		// Die gewollten Icon-Abst�nde
		x += iconInsets.left;
		y += iconInsets.top;
		// Rahmen muss ber�cksichtigt werden
		x += borderInsets.left;
		y += borderInsets.top;
		// Und zeichnen
		if (model.isSelected()) selectedIcon.paintIcon(c, g, x, y); else icon.paintIcon(c, g, x, y);
	}

}
