/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


public class ThemeLF_ComboBoxUI extends BasicComboBoxUI {
	private static final boolean DEBUG = false;

	// Diese Variablen m�ssen initialisiert sein wegen Button
	ThemeLF_Resource		resource 			= ThemeLF.getUI("ComboBox");
	ThemeLF_Border		border1 			= resource.getBorder("buttonBorder.normal");
	ThemeLF_Border		border2 			= resource.getBorder("buttonBorder.pressed");
	ThemeLF_Border		border3 			= resource.getBorder("editableButtonBorder.normal");
	ThemeLF_Border		border4 			= resource.getBorder("editableButtonBorder.pressed");
	ThemeLF_Border 		editorBorder		= resource.getBorder("editorBorder");
	IconUIResource		icon				= resource.getIcon("icon");
	ThemeLF_Disabler		disabler			= resource.getDisabler("disable");
	ColorUIResource		popupBackground	= resource.getColor("popupBackgroundColor");
	ColorUIResource		focusColor      		= resource.getColor("focusColor");
	boolean				useAlternateFocus	= resource.getBoolean("alternateFocus");
	
	
	class Button extends JButton implements javax.swing.border.Border {
		JComboBox combobox;
		
		
		public Button(JComboBox cbox)  {
			super();
			
			combobox = cbox;
			
			setIcon(icon);
			setBorder(this);
		}

		
		public Insets getBorderInsets(Component c)  {
			return combobox.isEditable() ? border3.getBorderInsets(c) : border1.getBorderInsets(c);
		}

		
		public boolean isBorderOpaque()  {
			return true;
		}

		
		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)  {
			if (combobox.isEditable()) 
				{
				if (((JButton)c).getModel().isPressed()) border4.paintBorder(c, g, x, y, w, h); else border3.paintBorder(c, g, x, y, w, h);
				}
			else
				{
				if (((JButton)c).getModel().isPressed()) border2.paintBorder(c, g, x, y, w, h); else border1.paintBorder(c, g, x, y, w, h);
				}
		}

	}

	
	class Editor extends BasicComboBoxEditor implements javax.swing.plaf.UIResource {
		
		
		public Editor()  {
			editor = new JTextField("", 9);
			editor.setBorder(editorBorder);
		}

	}

	
	class Popup extends BasicComboPopup implements UIResource {
		
		
		public Popup(JComboBox cBox)  {
			super(cBox);
			
			setBorder(null); // 'new ThreeDBorders.ComboBoxPopupBorder()' geht nicht ?!?
		}

		
		void configure()  {
			super.configureList();
			
			//System.out.println(ThemeLF_ComboBoxUI.this.popupBackground);
			
			list.setBackground(popupBackground);
		}

	}

	
	protected JButton createArrowButton()  {
		return new ThemeLF_ComboBoxUI.Button(comboBox);
	}

	
	protected ComboBoxEditor createEditor()  {
		return new ThemeLF_ComboBoxUI.Editor();
	}

	
	protected ComboPopup createPopup()  {
		ThemeLF_ComboBoxUI.Popup popup = new ThemeLF_ComboBoxUI.Popup(comboBox);
		
		popup.configure();
		
		return popup;
	}

	
	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_ComboBoxUI();
	}

	
	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 			= ThemeLF.getUI("ComboBox");
		border1 			= resource.getBorder("buttonBorder.normal");
		border2 			= resource.getBorder("buttonBorder.pressed");
		border3 			= resource.getBorder("editableButtonBorder.normal");
		border4 			= resource.getBorder("editableButtonBorder.pressed");
		icon				= resource.getIcon("icon");
		disabler			= resource.getDisabler("disable");
		popupBackground	= resource.getColor("popupBackgroundColor");
		
		comboBox.setRequestFocusEnabled(true);
		comboBox.setBorder(new ThemeLF_ComboBoxUI_Border());
	}

	
	public boolean isFocusTraversable()  {
		return false;
	}

	
	public void paint(Graphics g, JComponent c)  {
		super.paint(g, c);
		
		if (useAlternateFocus) 
			{
		      if (! comboBox.isEditable()) 
				{
				Rectangle r = rectangleForCurrentValue();
				paintCurrentValueBackground(g,r,false);
				paintCurrentValue(g,r,false);
				// Paint focus
				Insets insets = comboBox.getInsets();
				int width = (int)r.getWidth() - (insets.left + insets.right);
				int height = (int)r.getHeight() - (insets.top + insets.bottom);
				if (height <= 0 || width <= 0 ) return;
				int left = insets.left;
				int top = insets.top;
				int right = left + (width - 1);
				int bottom = top + (height - 1);
				if (comboBox.hasFocus()) 
					{
					g.setColor( focusColor );
					g.drawRect( left - 1, top + 1, width + 5, height + 1 );
					}
				}		
			} 
		else super.paint(g, c);
				
		if (! c.isEnabled()) disabler.paint(c, g, 0, 0, 0, 0);
	}

}
