/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;


public final class ThemeLF_Disabler implements Paintable {
	private static final boolean DEBUG = false;

	protected final static File 						basedir 			= new File(ThemeLF.getInstallationDirectory(), "data"+File.separator+"disabler");
	protected final static File 						classdir 			= new File(basedir, "classes");
	protected final static File 						sourcedir			= new File(basedir, "sources");
	protected final static DirectoryClassLoader 		classLoader 		= new DirectoryClassLoader(classdir);
	
	private Paintable 	painter 	= null;
	private String		name	= null;
	
	
	public ThemeLF_Disabler(String id)  {
		name = id;
		
		if (! id.equals("null"))
			{
			try 
				{
				Class c = classLoader.loadClass(id);
				painter = (Paintable) c.newInstance();
				}
			catch (Exception x) { System.err.println("ThemeLF_Disabler(): "+ x); System.exit(10); }
			}
	}

	
	static {
		classdir.mkdirs();
		sourcedir.mkdirs();
	}

	
	static final boolean check(String id)  {
		try 
			{
			Class c = classLoader.loadClass(id);
			return true;
			}
		catch (Exception x) { return false; }
	}

	
	protected static final void compile(String javac, File source, File dest, String classname)  {
		final Vector<String> alg = new Vector<String>();
		final Hashtable<String, String> data = new Hashtable<String, String>();
		
		System.out.println("ThemeLF_Disabler.compile(): compiling " + source);
		
		try
			{
			BufferedReader in = new BufferedReader(new FileReader(source));
			while (true) {
				String line = in.readLine();
				if (line == null) break;
				line = line.trim();
				if (line.startsWith("#")) continue;
				if (line.length()==0) continue;
				if (! line.startsWith("draw."))
					{
					int index = line.indexOf("=");
					if (index == -1) System.out.println("ThemeLF_Disabler.compile(): [Error 1] " + source +": missing '='");
					String tag = line.substring(0, index);
					String value = line.substring(index+1);
					data.put(tag, value);
					}
				else alg.addElement(line.substring(5));
				}
			in.close();
			}
		catch (Exception x) { System.err.println("ThemeLF_Disabler.compile(): [Error 2] " + x); System.exit(10); }
		
		try
			{
			PrintWriter out = new PrintWriter(new FileWriter(dest));
			BufferedReader in = new BufferedReader(new FileReader(source));
			
			out.println("import java.awt.*;");
			out.println("import javax.swing.*;");
			out.println("import javax.swing.border.*;");
			out.println("import de.hillenbrand.swing.plaf.themelf.*;");
			out.println("");
			out.println("public class " + classname + " implements Paintable {");
			out.println("");
			out.println("\tpublic void paint(Component c, Graphics g, int x, int y, int w, int h) {");
			for (int i=0; i<alg.size(); ++i) ThemeLF_Utils.generateCode(out, "\t\t", (String)alg.elementAt(i));
			out.println("\t}");
			out.println("");
			out.println("}");
			
			out.close();
			in.close();
			}
		catch (Exception x) { System.err.println("ThemeLF_Disabler.compile(): [Error 3] " + x); System.exit(10); }
		
		try
			{
			String cpath = System.getProperty("jedi.runfrom") != null 
				?  System.getProperty("user.home") + File.separator + "/jedi/classes" 
				: ThemeLF.getInstallationDirectory() + File.separator + "lib" + File.separator + "ThemeLF.jar";
				
			StringBuffer buffer = new StringBuffer();
			buffer.append(javac);
			buffer.append(" -O");
			buffer.append(" -d \"" + classdir + "\"");
			buffer.append(" -classpath \"" + cpath + "\"");
			buffer.append(" \"" + dest + "\"");
		
			Process process = Runtime.getRuntime().exec(ThemeLF_Border.toCommandArray(buffer));
			process.waitFor();
			
			if (process.exitValue() != 0) 
				{
				System.err.println("ThemeLF_Disabler.compile(): command was\n" + buffer);
				throw new RuntimeException("Compilation failed: " + process.exitValue());
				}
			}
		catch (Exception x) { System.err.println("ThemeLF_Disabler.compile(): [Error 4] " + x); System.exit(10); }
	}

	
	public static void compileAll(String javac)  {
		String files [] = basedir.list(new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return ! new File(dir, name).isDirectory();
			}
		});
		
		for (int i=0; i<files.length; ++i) 
			{
			File source = new File(basedir, files[i]);
			File dest = new File(sourcedir, files[i] + ".java");
			if (source.lastModified() > dest.lastModified()) compile(javac, source, dest, files[i]);
			}
	}

	
	public String getName()  {
		return name;
	}

	
	public void paint(Component c, Graphics g, int x, int y, int w, int h)  {
		if (painter != null) 
			{
			g.translate(x,y);
			painter.paint(c, g, x, y, w, h);
			g.translate(-x,-y);
			}
	}

	
	public String toString()  {
		return getClass().getName() + "[name="+name+"]";
	}

}
