/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


public class ThemeLF_ScrollPaneUI_ViewportBorder extends AbstractBorder implements UIResource {
	private static final boolean DEBUG = false;

	ThemeLF_Border 		treeBorder 			= ThemeLF.getUI("Tree").getBorder("border");
	ThemeLF_Border 		tableBorder 		= ThemeLF.getUI("Table").getBorder("border");
	ThemeLF_Border 		listBorder 			= ThemeLF.getUI("List").getBorder("border");
	ThemeLF_Border 		editorPaneBorder 	= ThemeLF.getUI("EditorPane").getBorder("border");
	ThemeLF_Border 		textAreaBorder 	= ThemeLF.getUI("TextArea").getBorder("border");
	
	
	public Insets getBorderInsets(Component c)  {
		Component view = ((JScrollPane)c).getViewport().getView();
		
		if (view != null && view instanceof JTree)
			{
			Insets i = treeBorder.getBorderInsets(c);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
			}
		else if (view != null && view instanceof JTable)
			{
			Insets i = tableBorder.getBorderInsets(c);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
			}
		else if (view != null && view instanceof JList)
			{
			Insets i = listBorder.getBorderInsets(c);
			return new Insets(i.top+1, i.left+1, i.bottom+1, i.right+1);
			}
		else if (view != null && view instanceof JEditorPane)
			{
			return editorPaneBorder.getBorderInsets(c);
			}
		else if (view != null && view instanceof JTextArea)
			{
			return textAreaBorder.getBorderInsets(c);
			}
		else return new Insets(0,0,0,0);
	}

	
	public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		Component view = ((JScrollPane)c).getViewport().getView();
		
		if (view != null && view instanceof JTree)
			{
			treeBorder.paintBorder(c, g, x, y, width, height);
			}
		else if (view != null && view instanceof JTable)
			{
			tableBorder.paintBorder(c, g, x, y, width, height);
			}
		else if (view != null && view instanceof JList)
			{
			listBorder.paintBorder(c, g, x, y, width, height);
			}
		else if (view != null &&  view instanceof JTextArea)
			{
			textAreaBorder.paintBorder(c, g, x, y, width, height);
			}
		else if (view != null && view instanceof JEditorPane)
			{
			editorPaneBorder.paintBorder(c, g, x, y, width, height);
			}
	}

}
