/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


public class ThemeLF_SplitPaneUI extends BasicSplitPaneUI {
	private static final boolean DEBUG = false;

	ThemeLF_Resource	resource 					= null;
	
	ThemeLF_Border		dividerBorderHorizontal	= null;
	ThemeLF_Border		dividerBorderVertical		= null;
	IconUIResource		dividerFillPattern			= null;
	int 						dividerBorderOffset			= 0;
	int						dividerSize					= 0;
	ThemeLF_Disabler		disabler 					= null;
	
	IconUIResource		handleIcon					= null;
	InsetsUIResource		handleInsets				= null;
	ColorUIResource		handleBackgroundColor	= null;
	ThemeLF_Border		handleBorder				= null;
	Insets					handleBorderInsets		= null;
	int 						handleOffset				= 0;
	boolean				drawHandle				= false;
	
	
	public class Divider extends BasicSplitPaneDivider {
		private static final int HEIGHT 	= 1;
		private static final int WIDTH	 	= 2;
		
		
		public Divider(BasicSplitPaneUI ui)  {
			super(ui);
		}

		
		public int getDividerSize()  {
			return splitPane.getOrientation() == JSplitPane.VERTICAL_SPLIT ? getHandleSize(HEIGHT) : getHandleSize(WIDTH);
		}

		
		public int getHandleSize(int i)  {
			if (drawHandle == false) return ThemeLF_SplitPaneUI.this.dividerSize;
			
			int result = 0;
			
			if (i == HEIGHT)
				{
				if (handleIcon != null) result += handleIcon.getIconHeight();
				result += handleInsets.top + handleInsets.bottom;
				result += handleBorderInsets.top + handleBorderInsets.bottom;
				}
			else
				{
				if (handleIcon != null) result += handleIcon.getIconWidth();
				result += handleInsets.left + handleInsets.right;
				result += handleBorderInsets.left + handleBorderInsets.right;
				}
			
			return result;
		}

		
		public void paint(Graphics g)  {
			super.paint(g);
			
			Dimension 	size 	= splitPane.getSize();
			final int 	OFFSET 	= splitPane.isOneTouchExpandable() ? 30 : handleOffset;
			
			if (splitPane.getOrientation() == JSplitPane.VERTICAL_SPLIT)
				{
				int handleWidth = getHandleSize(WIDTH);
				int handleHeight = getHandleSize(HEIGHT);
				Insets dividerBorderInsets = dividerBorderVertical.getBorderInsets(splitPane);
				// FillPattern
				if (dividerFillPattern != null)
					{
					Shape clip = g.getClip();
					g.setClip(new Rectangle(
						splitPane.isOneTouchExpandable() ? 30 : 0, 
						dividerBorderOffset+dividerBorderInsets.top, 
						size.width, 
						handleHeight-dividerBorderOffset-dividerBorderOffset-dividerBorderInsets.top-dividerBorderInsets.bottom));
					for (int y=dividerBorderOffset+dividerBorderInsets.top; y < handleHeight; y+=dividerFillPattern.getIconHeight())
						for (int x=splitPane.isOneTouchExpandable() ? 30 : 0; x<size.width; x+=dividerFillPattern.getIconWidth())
							dividerFillPattern.paintIcon(splitPane, g, x, y);
					g.setClip(clip);
					}
				// Rahmen zeichnen
				dividerBorderVertical.paintBorder(splitPane, g, 0, dividerBorderOffset, size.width, handleHeight-dividerBorderOffset-dividerBorderOffset);
				// Handle zeichnen
				if (drawHandle)
					{
					handleBorder.paintBorder(splitPane, g, OFFSET == 0 ? handleOffset : OFFSET, 0, handleWidth, handleHeight);
					g.setColor(handleBackgroundColor);
					g.fillRect(
						OFFSET+handleBorderInsets.left, 
						handleBorderInsets.top, 
						handleWidth-handleBorderInsets.left-handleBorderInsets.right, 
						handleHeight-handleBorderInsets.top-handleBorderInsets.bottom);
					// Icon malen
					if (handleIcon != null) handleIcon.paintIcon(splitPane, g, OFFSET+handleBorderInsets.left+handleInsets.left, handleBorderInsets.top+handleInsets.top);
					}
				}
			else // HORIZONTAL
				{
				int handleWidth = getHandleSize(WIDTH);
				int handleHeight = getHandleSize(HEIGHT);
				Insets dividerBorderInsets = dividerBorderHorizontal.getBorderInsets(splitPane);
				// FillPattern
				if (dividerFillPattern != null)
					{
					Shape clip = g.getClip();
					g.setClip(new Rectangle(
						dividerBorderOffset+dividerBorderInsets.left,
						splitPane.isOneTouchExpandable() ? 30 : 0, 
						handleWidth-dividerBorderOffset-dividerBorderOffset-dividerBorderInsets.left-dividerBorderInsets.right,
						size.height)); 
					for (int y=splitPane.isOneTouchExpandable() ? 30 : 0; y < size.height; y+=dividerFillPattern.getIconHeight())
						for (int x=dividerBorderOffset+dividerBorderInsets.left; x<handleWidth; x+=dividerFillPattern.getIconWidth())
							dividerFillPattern.paintIcon(splitPane, g, x, y);
					g.setClip(clip);
					}
				// Rahmen zeichnen
				dividerBorderHorizontal.paintBorder(splitPane, g, dividerBorderOffset, 0, handleWidth-dividerBorderOffset-dividerBorderOffset, size.height);
				// Handle zeichnen
				if (drawHandle)
					{
					handleBorder.paintBorder(splitPane, g, 0, OFFSET == 0 ? handleOffset : OFFSET, handleWidth, handleHeight);
					g.setColor(handleBackgroundColor);
					g.fillRect(
						handleBorderInsets.left,
						OFFSET+handleBorderInsets.top,  
						handleWidth-handleBorderInsets.left-handleBorderInsets.right, 
						handleHeight-handleBorderInsets.top-handleBorderInsets.bottom);
					// Icon malen
					if (handleIcon != null) handleIcon.paintIcon(splitPane, g, handleBorderInsets.left+handleInsets.left, OFFSET+handleBorderInsets.top+handleInsets.top);
					}
				}
			
			if (splitPane.isOneTouchExpandable())
				{
				if (leftButton != null) leftButton.repaint();
				if (rightButton != null) rightButton.repaint();
				}
		}

	}

	
	public BasicSplitPaneDivider createDefaultDivider()  {
		return new ThemeLF_SplitPaneUI.Divider(this);
	}

	
	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_SplitPaneUI();
	}

	
	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 					= ThemeLF.getUI("SplitPane");
		
		// Divider
		dividerBorderHorizontal	= resource.getBorder("dividerBorderHorizontal");
		dividerBorderVertical		= resource.getBorder("dividerBorderVertical");
		dividerFillPattern			= resource.getIcon("dividerFillPattern");
		dividerBorderOffset			= resource.getInteger("dividerBorderOffset");
		
		if (dividerFillPattern.getIconWidth() == -1) dividerFillPattern=null;
		
		// Handle
		drawHandle = resource.getBoolean("drawHandle");
		if (drawHandle)
			{
			handleIcon					= resource.getIcon("handleIcon");
			handleInsets				= resource.getInsets("handleInsets");
			handleBackgroundColor	= resource.getColor("handleBackgroundColor");
			handleBorder				= resource.getBorder("handleBorder");
			handleOffset				= resource.getInteger("handleOffset");
		
			if (handleIcon.getIconWidth() == -1) handleIcon=null;
			handleBorderInsets = handleBorder.getBorderInsets(splitPane);
			}
		else dividerSize = resource.getInteger("dividerSize");
	}

	
	public void paint(Graphics g, JComponent c)  {
		super.paint(g, c);
		
		if (! c.isEnabled()) disabler.paint(c, g, 0, 0, 0, 0);
	}

}
