/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


public class ThemeLF_TabbedPaneUI extends BasicTabbedPaneUI {
	private static final boolean DEBUG = false;

	private ThemeLF_Resource 	resource 							= null;
	private ColorUIResource		backgroundColor					= null;
	private ColorUIResource		darkEdgeColor					= null;
	private ColorUIResource		focusColor						= null;
	private ColorUIResource		lightEdgeColor					= null;
	private ColorUIResource		selectedTabBackgroundColor	= null;
	private ColorUIResource		tabBackgroundColor				= null;
	private ThemeLF_Border	contentBorder					= null;
	private ThemeLF_Border	tabBorder							= null;
	private ThemeLF_Border	selectedTabBorder				= null;
	private Insets				contentBorderInsets				= null;
	private Insets				tabBorderInsets					= null;
	private Insets				selectedTabBorderInsets		= null;
	private ThemeLF_Disabler 	disabler 							= null;
	
	
	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_TabbedPaneUI();
	}

	
	public void installUI(JComponent c)  {
		super.installUI(c);
			
		resource 						= ThemeLF.getUI("TabbedPane");
		backgroundColor				= resource.getColor("backgroundColor");
		contentBorder					= resource.getBorder("contentBorder");
		darkEdgeColor					= resource.getColor("darkEdgeColor");
		focusColor						= resource.getColor("focusColor");
		lightEdgeColor					= resource.getColor("lightEdgeColor");
		selectedTabBackgroundColor	= resource.getColor("selectedTabBackgroundColor");
		tabBackgroundColor			= resource.getColor("tabBackgroundColor");
		tabBorder						= resource.getBorder("tabBorder");
		selectedTabBorder				= resource.getBorder("selectedTabBorder");
		
		contentBorderInsets			= contentBorder.getBorderInsets(tabPane);
		tabBorderInsets				= tabBorder.getBorderInsets(tabPane);
		selectedTabBorderInsets		= selectedTabBorder.getBorderInsets(tabPane);
	}

	
	public void paint(Graphics g, JComponent c)  {
		int 				tabPlacement 	= tabPane.getTabPlacement();
		Insets 		insets 				= c.getInsets(); 
		Dimension 	size 				= c.getSize();
		int 				totalTabWidth 	= 0;
		int 				totalTabHeight 	= 0;
		
		// Paint the background for the tab area
		if ( tabPane.isOpaque() ) 
			{
			g.setColor(c.getBackground());
			switch (tabPlacement) 
				{
				case LEFT:
					totalTabWidth = calculateTabAreaWidth(tabPlacement, runCount, maxTabWidth);
					g.fillRect(insets.left, insets.top, totalTabWidth, size.height - insets.bottom - insets.top );
					break;
				case BOTTOM:
					totalTabHeight = calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
					g.fillRect( insets.left, size.height - insets.bottom - totalTabHeight, size.width - insets.left - insets.right, totalTabHeight );
					break;
				case RIGHT:
					totalTabWidth = calculateTabAreaWidth(tabPlacement, runCount, maxTabWidth);
					g.fillRect(size.width - insets.right - totalTabWidth, insets.top, totalTabWidth, size.height - insets.top - insets.bottom);
					break;
				default: // TOP
					totalTabHeight = calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
					g.fillRect( insets.left, insets.top, size.width - insets.right - insets.left, totalTabHeight);
				}
			}
		
		super.paint(g, c);
		
		if (! c.isEnabled()) disabler.paint(c, g, 0, 0, 0, 0);
	}

	
	protected void paintContentBorder(Graphics g, int tabPlacement, int selectedIndex)  {
		int width = tabPane.getWidth();
		int height = tabPane.getHeight();
		
		Insets insets = tabPane.getInsets();
		
		int x = insets.left;
		int y = insets.top;
		int w = width - insets.right - insets.left;
		int h = height - insets.top - insets.bottom;
		
		switch(tabPlacement)
			{
			case LEFT:
				x += calculateTabAreaWidth(tabPlacement, runCount, maxTabWidth);
				w -= (x - insets.left);
				paintContentBorder_Left(g, selectedIndex, x, y, w, h);
				break;
			case RIGHT:
				w -= calculateTabAreaWidth(tabPlacement, runCount, maxTabWidth);
				paintContentBorder_Right(g, selectedIndex, x, y, w, h);
				break;            
			case BOTTOM: 
				h -= calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
				paintContentBorder_Bottom(g, selectedIndex, x, y, w, h);
				break;
			case TOP:
			default:
				y += calculateTabAreaHeight(tabPlacement, runCount, maxTabHeight);
				h -= (y - insets.top);
				paintContentBorder_Top(g, selectedIndex, x, y, w, h);
			}         
	}

	
	protected void paintContentBorder_Bottom(Graphics g, int selectedIndex, int x, int y, int w, int h)  {
		final Shape oldClip = g.getClip();
		
		if (selectedIndex < rects.length)
			{
			final Rectangle selRect = rects[selectedIndex];
			final Area a = new Area(new Rectangle(x,y,w,h));
			a.subtract(new Area(new Rectangle(x+selRect.x, y+h-contentBorderInsets.bottom, selRect.width, contentBorderInsets.bottom)));
			g.setClip(a);
			contentBorder.paintBorder(tabPane, g, x, y, w, h);
			g.setClip(oldClip);
			}
	}

	
	protected void paintContentBorder_Left(Graphics g, int selectedIndex, int x, int y, int w, int h)  {
		final Shape oldClip = g.getClip();
		
		if (selectedIndex < rects.length)
			{
			final Rectangle selRect = rects[selectedIndex];
			final Area a = new Area(new Rectangle(x,y,w,h));
			a.subtract(new Area(new Rectangle(x, y+selRect.y, contentBorderInsets.left, selRect.height)));
			g.setClip(a);
			contentBorder.paintBorder(tabPane, g, x, y, w, h);
			g.setClip(oldClip);
			}
	}

	
	protected void paintContentBorder_Right(Graphics g, int selectedIndex, int x, int y, int w, int h)  {
		final Shape oldClip = g.getClip();
		
		if (selectedIndex < rects.length)
			{
			final Rectangle selRect = rects[selectedIndex];
			final Area a = new Area(new Rectangle(x,y,w,h));
			a.subtract(new Area(new Rectangle(x+w-contentBorderInsets.right, y+selRect.y, contentBorderInsets.right, selRect.height)));
			g.setClip(a);
			contentBorder.paintBorder(tabPane, g, x, y, w, h);
			g.setClip(oldClip);
			}
	}

	
	protected void paintContentBorder_Top(Graphics g, int selectedIndex, int x, int y, int w, int h)  {
		final Shape oldClip = g.getClip();
		
		if (selectedIndex < rects.length)
			{
			final Rectangle selRect = rects[selectedIndex];
			final Area a = new Area(new Rectangle(x,y,w,h));
			a.subtract(new Area(new Rectangle(x+selRect.x, y, selRect.width, contentBorderInsets.top)));
			g.setClip(a);
			contentBorder.paintBorder(tabPane, g, x, y, w, h);
			g.setClip(oldClip);
			}
	}

	
	protected void paintFocusIndicator(Graphics g, int tabPlacement, Rectangle [] rects, int tabIndex, Rectangle iconRect, Rectangle textRect, boolean isSelected)  {
		if (! isSelected) return;
		if (! tabPane.hasFocus()) return;
		
		if (textRect.width > 0 && iconRect.width > 0)
			{
			final Rectangle drawRect = iconRect.union(textRect);
			g.setColor(focusColor);
			g.drawRect(drawRect.x - 1, drawRect.y - 1, drawRect.width + 1, drawRect.height + 1);
			}
		else if (textRect.width > 0) // Text umranden 
			{
			g.setColor(focusColor);
			g.drawRect(textRect.x - 1, textRect.y - 1, textRect.width + 1, textRect.height + 1);
			}
		else if (iconRect.width > 0) // Icon umranden
			{
			g.setColor(focusColor);
			g.drawRect(iconRect.x - 1, iconRect.y - 1, iconRect.width + 1, iconRect.height + 1);
			}
		// else System.err.println("ThreeD_TabbedPaneUI.paintFocusIndicator(): TabbedPane without text and icon!");
	}

	
	protected void paintTabBackground(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected)  {
		g.setColor(isSelected ? selectedTabBackgroundColor : tabBackgroundColor);
		
		x += tabBorderInsets.left;
		y += tabBorderInsets.top;
		w -= tabBorderInsets.left - tabBorderInsets.right;
		h -= tabBorderInsets.top - tabBorderInsets.bottom;
		
		g.fillRect(x,y,w-1,h-1);
	}

	
	protected void paintTabBorder(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected)  {
		int bottom = y + (h-1);
		int right = x + (w-1);
		
		switch (tabPlacement)
			{
			case LEFT:
				paintTabBorder_Left(tabIndex, g, x, y, w, h, bottom, right, isSelected);
				break;
			case BOTTOM:
				paintTabBorder_Bottom(tabIndex, g, x, y, w, h, bottom, right, isSelected);
				break;
			case RIGHT:
				paintTabBorder_Right(tabIndex, g, x, y, w, h, bottom, right, isSelected);
				break;
			case TOP:
			default:
				paintTabBorder_Top(tabIndex, g, x, y, w, h, bottom, right, isSelected);
		 	}
		 
	}

	
	protected void paintTabBorder_Bottom(int tabIndex, Graphics g, int x, int y, int w, int h, int btm, int rght, boolean isSelected)  {
		Shape oldClip = g.getClip();
		
		Rectangle r = new Rectangle(x,y,w,h);
		g.setClip(r);
		
		if (isSelected)
			selectedTabBorder.paintBorder(tabPane, g, x, y-contentBorderInsets.bottom-selectedTabBorderInsets.top, w, h+contentBorderInsets.bottom+selectedTabBorderInsets.top);
		else
			tabBorder.paintBorder(tabPane, g, x, y-contentBorderInsets.bottom-tabBorderInsets.top, w, h+contentBorderInsets.bottom+tabBorderInsets.top);
		
		g.setClip(oldClip);
	}

	
	protected void paintTabBorder_Left(int tabIndex, Graphics g, int x, int y, int w, int h, int btm, int rght, boolean isSelected)  {
		Shape oldClip = g.getClip();
		
		Rectangle r = new Rectangle(x,y,w,h);
		g.setClip(r);
		
		if (isSelected)
			selectedTabBorder.paintBorder(tabPane, g, x, y, w+contentBorderInsets.left+selectedTabBorderInsets.right, h);
		else
			tabBorder.paintBorder(tabPane, g, x, y, w+contentBorderInsets.left+tabBorderInsets.right, h);
		
		g.setClip(oldClip);
	}

	
	protected void paintTabBorder_Right(int tabIndex, Graphics g, int x, int y, int w, int h, int btm, int rght, boolean isSelected)  {
		Shape oldClip = g.getClip();
		
		Rectangle r = new Rectangle(x,y,w,h);
		g.setClip(r);
		
		if (isSelected)
			selectedTabBorder.paintBorder(tabPane, g, x-contentBorderInsets.right-selectedTabBorderInsets.left, y, w+contentBorderInsets.right+selectedTabBorderInsets.left, h);
		else
			tabBorder.paintBorder(tabPane, g, x-contentBorderInsets.right-tabBorderInsets.left, y, w+contentBorderInsets.right+tabBorderInsets.left, h);
			
		g.setClip(oldClip);
	}

	
	protected void paintTabBorder_Top(int tabIndex, Graphics g, int x, int y, int w, int h, int btm, int rght, boolean isSelected)  {
		Shape oldClip = g.getClip();
		
		Rectangle r = new Rectangle(x,y,w,h);
		g.setClip(r);
		
		if (isSelected)
			selectedTabBorder.paintBorder(tabPane, g, x, y, w, h+contentBorderInsets.top+selectedTabBorderInsets.bottom);
		else
			tabBorder.paintBorder(tabPane, g, x, y, w, h+contentBorderInsets.top+tabBorderInsets.bottom);
		
		g.setClip(oldClip);
	}

}
