/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.text.*;


public class ThemeLF_TextFieldUI extends BasicTextFieldUI {
	private static final boolean DEBUG = false;

	JComponent			component 		= null;
	ThemeLF_Resource	resource		= null;
	ThemeLF_Disabler 		disabler		= null;
	
	
	public ThemeLF_TextFieldUI(JComponent c)  {
		component = c;
	}

	
	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_TextFieldUI(c);
	}

	
	private void editableChanged(JComponent c, boolean editable)  {
		Color background = c.getBackground();
		
		if (editable == false)
			{
			if (background instanceof UIResource) 
				{
				c.setBackground(UIManager.getColor("TextField.control"));
				}
			} 
		else 
			{
			if (background instanceof UIResource)
				{
				c.setBackground(UIManager.getColor("TextField.background"));
				}
			}
	}

	
	private void enableChanged(JComponent c, boolean enable)  {
		if (enable == false) 
			{
			c.setBackground(resource.getColor("disableColor"));
			}
		else
			{
			c.setBackground(UIManager.getColor("TextField.background"));
			}
	}

	
	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 	= ThemeLF.getUI("TextField");
		disabler 	= resource.getDisabler("disable");
		
		if (disabler.getName().equalsIgnoreCase("null")) 
			{
			editableChanged(c, ((JTextComponent)c).isEditable());
			enableChanged(c, ((JTextComponent)c).isEnabled());
			}
	}

	
	public void paintSafely(Graphics g)  {
		super.paintSafely(g);
		
		if (! component.isEnabled()&!disabler.getName().equalsIgnoreCase("null")) disabler.paint(component, g, 0, 0, 0, 0);
	}

	
	public void propertyChange(PropertyChangeEvent e)  {
		if (disabler != null)
			{
			if (! disabler.getName().equalsIgnoreCase("null")) return;
			}
		
		if (e.getPropertyName().equals("editable"))
			{
			JComponent source = (JComponent) e.getSource();
			Color background = source.getBackground();
			boolean editable = ((Boolean) e.getNewValue()).booleanValue();
			editableChanged(source, editable);
			}
		
		if (e.getPropertyName().equals("enabled"))
			{
			JComponent source = (JComponent) e.getSource();
			Color background = source.getBackground();
			boolean enabled = ((Boolean) e.getNewValue()).booleanValue();
			enableChanged(source, enabled);
			}
		
		super.propertyChange(e);
	}

}
