/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


public class ThemeLF_ToggleButtonUI extends BasicToggleButtonUI {
	private static final boolean DEBUG = false;

	ThemeLF_Resource	resource 					= null;
	ColorUIResource		focusColor					= null;
	ColorUIResource		pressedBackgroundColor	= null;
	ThemeLF_Disabler 		disabler 					= null;
	
	
	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_ToggleButtonUI();
	}

	
	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 					= ThemeLF.getUI("Button");
		focusColor					= resource.getColor("focusColor");
		pressedBackgroundColor	= resource.getColor("pressedBackgroundColor");
		disabler 					= resource.getDisabler("disable");
	}

	
	public void paint(Graphics g, JComponent c)  {
		super.paint(g, c);
		
		if (! c.isEnabled()) disabler.paint(c, g, 0, 0, 0, 0);
	}

	
	protected void paintButtonPressed(Graphics g, AbstractButton b)  {
		Dimension size = b.getSize();
		
		g.setColor(pressedBackgroundColor);
		g.fillRect(1,1,size.width-2, size.height-2);
		
		super.paintButtonPressed(g, b);
	}

	
	protected void paintFocus(Graphics g, AbstractButton b, Rectangle viewRect, Rectangle textRect, Rectangle iconRect)  {
		Rectangle focusRect = new Rectangle();
		String text = b.getText();
		
		if (text != null && !text.equals("")) // If there is text
			{
			if (b.getIcon() == null)
				{
				focusRect.setBounds(textRect);
				}
			else
				{
				focusRect.setBounds(iconRect.union(textRect));
				}
			}
		else if (b.getIcon() != null) // If there is an icon and no text
			{
			focusRect.setBounds(iconRect);
			}
		
		g.setColor(focusColor);
		g.drawRect(focusRect.x - 1, focusRect.y - 1, focusRect.width + 1, focusRect.height + 1);
	}

}
