/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.swing.plaf.themelf;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;


public final class ThemeLF_Utils {
	private static final boolean DEBUG = false;

	
	
	public static final void generateCode(PrintWriter out, String prefix, String code)  {
		int index = code.indexOf("=");
		
		String command = code.substring(0, index);
		String args = code.substring(index+1);
		
		if (command.equals("color"))
			{
			if (args.indexOf(",") != -1)
				{
				out.println(prefix + "g.setColor(new Color(" + args + "));");
				}
			else
				{
				out.println(prefix +  "g.setColor(ThemeLF_Prefs.getDefaultColor(\"" + args + "\"));");
				}
			}
		else if (command.equals("line"))
			{
			if (args.indexOf(",") != -1)
				{
				out.println(prefix + "g.drawLine(" + args + ");");
				}
			else throw new RuntimeException("Not yet supported: " + code);
			}
		else if (command.equals("point"))
			{
			if (args.indexOf(",") != -1)
				{
				out.println(prefix + "g.drawLine(" + args + "," + args +");");
				}
			else throw new RuntimeException("Not yet supported: " + code);
			}
		else if (command.equals("rect"))
			{
			if (args.indexOf(",") != -1)
				{
				out.println(prefix + "g.drawRect(" + args + ");");
				}
			else throw new RuntimeException("Not yet supported: " + code);
			}
		else if (command.equals("fillRect"))
			{
			if (args.indexOf(",") != -1)
				{
				out.println(prefix + "g.fillRect(" + args + ");");
				}
			else throw new RuntimeException("Not yet supported: " + code);
			}
		else if (command.equals("if"))
			{
			out.println(prefix + "if (" + args + ") {");
			}
		else if (command.equals("endif"))
			{
			out.println(prefix + "}");
			}
		else if (command.equals("code"))
			{
			out.println(prefix + args);
			}
		else throw new RuntimeException("Unknown command: " + code);
	}

	
	public static final IconUIResource loadIcon(String s)  {
		ImageIcon icon = new ImageIcon(ThemeLF.getInstallationDirectory() 
			+ File.separator + "data" + File.separator + "icons" 
			+ File.separator + s.replace('/', File.separatorChar) );
		
		return new IconUIResource(icon);
	}

	
	public static final ColorUIResource parseColor(String s)  {
		if (s.indexOf(",") != -1)
			{
			StringTokenizer tok = new StringTokenizer(s, ", ");
			int r = Integer.parseInt(tok.nextToken());
			int g = Integer.parseInt(tok.nextToken());
			int b = Integer.parseInt(tok.nextToken());
			return new ColorUIResource(r,g,b);
			}
		else return ThemeLF_Prefs.getDefaultColor(s);
	}

	
	public static final Dimension parseDimension(String s)  {
		if (s.indexOf(",") != -1)
			{
			StringTokenizer tok = new StringTokenizer(s, ", ");
			int i1 = Integer.parseInt(tok.nextToken());
			int i2 = Integer.parseInt(tok.nextToken());
			return new Dimension(i1,i2);
			}
		else
			{
			System.err.println("Noch nicht implementiert: Insets ausw�hlen ( " + s + ")");
			return new Dimension(10,10);
			}
	}

	
	public static final ThemeLF_Disabler parseDisabler(String s)  {
		if (s.equals("null") || ThemeLF_Disabler.check(s))
			{
			return new ThemeLF_Disabler(s);
			}
		else 
			{ 
			ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
			if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseDisabler(): cannot find Defaults file");
			return resource.getDisabler(s);
			}
	}

	
	public static final FontUIResource parseFont(String s)  {
		if (s.indexOf(",") != -1)
			{
			StringTokenizer tok = new StringTokenizer(s, ",");
			String name = tok.nextToken();
			int size = Integer.parseInt(tok.nextToken());
			return new FontUIResource(name, Font.PLAIN, size);
			}
		else return ThemeLF_Prefs.getDefaultFont(s);
	}

	
	public static final InsetsUIResource parseInsets(String s)  {
		if (s.indexOf(",") != -1)
			{
			StringTokenizer tok = new StringTokenizer(s, ", ");
			int i1 = Integer.parseInt(tok.nextToken());
			int i2 = Integer.parseInt(tok.nextToken());
			int i3 = Integer.parseInt(tok.nextToken());
			int i4 = Integer.parseInt(tok.nextToken());
			return new InsetsUIResource(i1,i2,i3,i4);
			}
		else
			{
			ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
			if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseColors(): cannot find Defaults file");
			return resource.getInsets(s);
			}
	}

	
	public static void showError(String s)  {
		JOptionPane.showMessageDialog(null, s, "Error", JOptionPane.ERROR_MESSAGE);
	}

	
	public static void showException(Exception x)  {
		JOptionPane.showMessageDialog(null, x.toString(), "Error", JOptionPane.ERROR_MESSAGE);
	}

}
